/*************************************************************************
 * HeXHuB C/C++ Plugin SDK files (C) Copyright 2006 - 2007 Morten Larsen *
 * HeXHuB is (C) Copyright Albu Christian Laurentiu                      *
 *************************************************************************/

#ifndef HEXHUB_H
#define HEXHUB_H

//To use with GetOpStatus() or SetOpStatus() functions
#define HEX_OPSTATUS_ERR_NOT_FOUND -1
#define HEX_OPSTATUS_NORMAL_USER    0
#define HEX_OPSTATUS_PUBLIC_OP      1
#define HEX_OPSTATUS_HIDDEN_OP      2

#define BIT_0 0x00000001
#define BIT_1 0x00000002
#define BIT_2 0x00000004
#define BIT_3 0x00000008
#define BIT_4 0x00000010
#define BIT_5 0x00000020
#define BIT_6 0x00000040
#define BIT_7 0x00000080
#define BIT_8 0x00000100
#define BIT_9 0x00000200

#define BIT64_0  0x0000000000000001
#define BIT64_1  0x0000000000000002
#define BIT64_2  0x0000000000000004
#define BIT64_3  0x0000000000000008
#define BIT64_4  0x0000000000000010
#define BIT64_5  0x0000000000000020
#define BIT64_6  0x0000000000000040
#define BIT64_7  0x0000000000000080
#define BIT64_8  0x0000000000000100
#define BIT64_9  0x0000000000000200
#define BIT64_10 0x0000000000000400
#define BIT64_11 0x0000000000000800
#define BIT64_12 0x0000000000001000
#define BIT64_13 0x0000000000002000
#define BIT64_14 0x0000000000004000
#define BIT64_15 0x0000000000008000
#define BIT64_16 0x0000000000010000
#define BIT64_17 0x0000000000020000
#define BIT64_18 0x0000000000040000


#define HEXHUBAPI __stdcall

typedef struct _userFloodInfo {
	long	lasttick;	//number of seconds since hub was started
	short	flcnt1;		//command counter, current user
	short	flcnt2;		//command counter, all users
	char	flag1;		//command flags
} userFloodInfo;

typedef struct _floodDefInfo {
	char *	lpszCmdName;	//command name
	void*	cmdproc;	    //address of procedure that is called when specified command is received
	long	floodFlags;	    //flags used by flood check procedures
				//bit  0
				//bit  1
				//bit  2 => bits 0,1,2 are used as follows:
					// 210
					// 000 = /session
					// 001 = /second
					// 010 = /minute
					// 011 = /hour
					// 100 = /day
				//bit  3
				//bit  4
				//bit  5 => bits 3,4,5 are used as follows:
					// 543
					// 000 = /session
					// 001 = /second
					// 010 = /minute
					// 011 = /hour
					// 100 = /day
				//bit  8 => keep only the last of a command chain
				//bit  9 => restriction enabled: "allow only ... from one"
				//bit 10 => restriction enabled: "allow only ... from all"
				//bit 11 => reserved
				//bit 12 => if flood is detected, user is delayed
				//bit 13 => if flood is detected, command is ignored
				//bit 14 => if flood is detected, a notification message will be sent
				//bit 15 => if flood is detected, user will be disconnected
				//bit 16 => do Not combine messages
	short	flcnt1;		//maximum allowed from one user
	short	flcnt2;		//maximum allowed from all users
	long	counter;	//total number of commands received from all users (for !stats command)
	long	cmdFlags;	//flags related to specified command
	long	cmdParam1;	//parameters specific to this command
	long	cmdParam2;	//parameters specific to this command
	long	lastTick;	//number of seconds since hub was started
	short	_flcnt1;	//command counter, all users
	short	_flcnt2;	//number of times when flood was detected from all users
} floodDefInfo;

typedef struct _ipInfo {
	long	lastIp;		//IP number
	short	timer1;
	short	timer2;
	short	timer3;
	short	timer4;
	short	timer5;		//timers used to check for reconnect flood
	char	fcount;		//number of reconnects
	char	flags1;		//flags
				//bit 0 => reserved
				//bit 1 => reserved
				//bit 2 => used by DDoS filter
				//bit 3 => IP was checked for public proxy
				//bit 4 => IP is a public proxy
				//bit 5 => from this IP a user was disconnected because of incorrect settings for active mode
				//bit 6 => this IP belongs to a faker and a notification message was sent
				//bit 7 => reserved
	long	lastconn;	//last connection time in seconds relative to hub startup
	long	tc_save;	//reserved
	char	langtmp;	//language index in language table
	char	invpasvd;	//number of invalid passshorts sent
	char	timeskicked;	//number of kicks
	char	modetmp;	//last chat filter's index (used by older versions)
	char	country;	//country name index
	char	flagextra;	//flags
				//bit 0 => country ID was detected
				//bit 1 => user was banned for forbidden files in share, share size must change
	char	share_check;	//checksum for share size
	char	macaddr[6];	//MAC address
	char	reserved[3];	//reserved
} ipInfo;

typedef struct _profileInfo {
	char	rights[8];	//rights
	long	minAccess;	//minimum access
	long	maxAccess;	//maximum access
	long	accessIncrease;	//increment
	long	availability;	//availability in hours (0 = permanent)
} profileInfo;

typedef struct _accountInfo {
	long	reserved1;  	    //reserved
	char	name[40];   	    //user name
	short	profileId;  	    //profile identifier
	short	profileReset;	    //identifier of the profile it resets to
	long	userId;	    	    //user identifier
	char	rights[8];  	    //rights
	long	access;	            //access level
	long	maxAccess;        	//maximum access for current profile
	long	accessTime;     	//timer increment
	long	availability;   	//account availability
	long	maxClones;       	//maximum clones allowed (if the account is for an IP)
	char	language[2];    	//language identifier
	short	mode;	        	//last chat filter used
	char	passunsigned[40];	//passshort
	char	ipInformation;	    //1 = IP/IP range, 2 = ISP, 3 = MAC
	long	IP1;		        //first IP in range
	long	IP2;		        //last IP in range
	char	ipstr[40];	        //string containing IP, ISP or MAC (as ipInformation specifies)
	long	reserved2;	        //reserved
	char	creator[40];	    //nickname of the admin who created this account
	char	reserved3[8];	    //reserved, date+time
	char	modifier[40];	    //nickname of the last admin who modified this account
	char	reserved4[8];	    //reserved, date+time
	char	reserved5[50];	    //reserved
	char	reason[220];	    //reason or comment
	char	reserved6[62];	    //reserved
} accountInfo;

typedef FARPROC (HEXHUBAPI *lpfnGetFunction)(void *);
typedef bool    (HEXHUBAPI *lpfnSendTo)(long, const char*, int);
typedef void    (HEXHUBAPI *lpfnSendToAll)(const char*, int);
typedef long    (HEXHUBAPI *lpfnRegisterCommand)(const char*, void *);
typedef long    (HEXHUBAPI *lpfnRegisterHelpString)(const char *, const char *, long);
typedef long    (HEXHUBAPI *lpfnIdFromNick)(const char *);
typedef long    (HEXHUBAPI *lpfnGetUserCount)(void);
typedef long    (HEXHUBAPI *lpfnGetUserIds)(long *, long);
typedef bool    (HEXHUBAPI *lpfnClientNickFromId)(long, char *);
typedef long    (HEXHUBAPI *lpfnGetVersion)(void);
typedef void    (HEXHUBAPI *lpfnSendNotificationMessage)(const char *);
typedef bool    (HEXHUBAPI *lpfnNickFromId)(long, char *);
typedef long    (HEXHUBAPI *lpfnGetInfo)(long, char *);
typedef long    (HEXHUBAPI *lpfnGetFullInfo)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetInfo)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetDescription)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetDescription)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetTag)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetTag)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetConnection)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetConnection)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetMail)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetMail)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetShareString)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetShareString)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetIP)(long);
typedef long    (HEXHUBAPI *lpfnGetIPString)(long, char *);
typedef long    (HEXHUBAPI *lpfnGetISPName)(long, char *);
typedef long    (HEXHUBAPI *lpfnGetOpStatus)(long);
typedef bool    (HEXHUBAPI *lpfnSetOpStatus)(long, long);
typedef long    (HEXHUBAPI *lpfnGetLanguage)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetLanguage)(long, const char *);
typedef long    (HEXHUBAPI *lpfnGetMode)(long, char *);
typedef bool    (HEXHUBAPI *lpfnSetMode)(long, const char *);
typedef long    (HEXHUBAPI *lpfnModeTransform)(const char *, const char *, char*);
typedef long    (HEXHUBAPI *lpfnGetMastersCount)(void);
typedef long    (HEXHUBAPI *lpfnGetMastersIds)(long *, long);
typedef bool    (HEXHUBAPI *lpfnGetIpCacheInfo)(long, ipInfo *);
typedef void    (HEXHUBAPI *lpfnSetIpCacheInfo)(ipInfo *);
typedef long    (HEXHUBAPI *lpfnGetAccessLevel)(long);
typedef bool    (HEXHUBAPI *lpfnSetAccessLevel)(long, long);
typedef bool    (HEXHUBAPI *lpfnGetRights)(long, long long *);
typedef bool    (HEXHUBAPI *lpfnSetRights)(long, long long *);
typedef long    (HEXHUBAPI *lpfnGetProfileId)(long);
typedef long    (HEXHUBAPI *lpfnGetAccountId)(long);
typedef void    (HEXHUBAPI *lpfnCreateAccount)(const char*, const char *, const char *, const char *, long);
typedef bool    (HEXHUBAPI *lpfnDeleteAccount)(long);
typedef bool    (HEXHUBAPI *lpfnMoveAccount)(long, long, long);
typedef bool    (HEXHUBAPI *lpfnSetProfile)(long, const char *, profileInfo *, const char *, const char *);
typedef bool    (HEXHUBAPI *lpfnGetProfile)(long, char *, profileInfo *, char *, char *);
typedef long    (HEXHUBAPI *lpfnGetNextProfile)(long);
typedef bool    (HEXHUBAPI *lpfnGetAccount)(long, accountInfo *);
typedef bool    (HEXHUBAPI *lpfnSetAccount)(long, accountInfo *);
typedef long    (HEXHUBAPI *lpfnGetNextAccount)(long, long);
typedef long    (HEXHUBAPI *lpfnAllocUserData)(void);
typedef bool    (HEXHUBAPI *lpfnFreeUserData)(long);
typedef long    (HEXHUBAPI *lpfnGetUserData)(long, long);
typedef bool    (HEXHUBAPI *lpfnSetUserData)(long, long, unsigned char);
typedef bool    (HEXHUBAPI *lpfnUnregisterCommand)(long);
typedef bool    (HEXHUBAPI *lpfnUnregisterHelpString)(long);
typedef long    (HEXHUBAPI *lpfnRegisterUserCommand)(const char *);
typedef bool    (HEXHUBAPI *lpfnUnregisterUserCommand)(long);
typedef long    (HEXHUBAPI *lpfnRegisterChatFilter)(const char *, FARPROC, const char *);
typedef bool    (HEXHUBAPI *lpfnUnregisterChatFilter)(long);
typedef long    (HEXHUBAPI *lpfnRegisterBot)(const char *, FARPROC);
typedef bool    (HEXHUBAPI *lpfnUnregisterBot)(long);
typedef bool    (HEXHUBAPI *lpfnCallHubCommand)(long, const char *);
typedef bool    (HEXHUBAPI *lpfnProcessUserTraffic)(long, FARPROC, long, long);
typedef bool    (HEXHUBAPI *lpfnDisableOutputToUser)(long, FARPROC, bool);
typedef void    (HEXHUBAPI *lpfnAcceptUser)(int, long);
typedef long    (HEXHUBAPI *lpfnGetTopic)(char *, long);
typedef void    (HEXHUBAPI *lpfnSetTopic)(const char *);
typedef long    (HEXHUBAPI *lpfnGetMOTD)(char *, long);
typedef void    (HEXHUBAPI *lpfnSetMOTD)(const char *);
typedef long    (HEXHUBAPI *lpfnGetHubUptime)(void);
typedef long    (HEXHUBAPI *lpfnGetHublistEntry)(long, char *, long);
typedef long    (HEXHUBAPI *lpfnRegisterBotEx)(const char *, FARPROC, long, long, long);

#if (HEXHUB_VERSION >= 320)
typedef bool    (HEXHUBAPI *lpfnDisconnectUser)(long);
#endif

#if defined(HAVE_HEXHUB_CPP_UTILS) && defined(__cplusplus)

class HexHubFunctions {
public:
	static lpfnGetFunction               GetFunction;
	static lpfnSendTo                    SendTo;
	static lpfnSendToAll                 SendToAll;
	static lpfnRegisterCommand           RegisterCommand;
	static lpfnRegisterHelpString        RegisterHelpString;
	static lpfnIdFromNick                IdFromNick;
	static lpfnGetUserCount              GetUserCount;
	static lpfnGetUserIds                GetUserIds;
	static lpfnClientNickFromId          ClientNickFromId;
	static lpfnGetVersion                GetVersion;
	static lpfnSendNotificationMessage   SendNotificationMessage;
	static lpfnNickFromId                NickFromId;
	static lpfnGetInfo                   GetInfo;
	static lpfnGetFullInfo               GetFullInfo;
	static lpfnSetInfo                   SetInfo;
	static lpfnGetDescription            GetDescription;
	static lpfnSetDescription            SetDescription;
	static lpfnGetTag                    GetTag;
	static lpfnSetTag                    SetTag;
	static lpfnGetConnection             GetConnection;
	static lpfnSetConnection             SetConnection;
	static lpfnGetMail                   GetMail;
	static lpfnSetMail                   SetMail;
	static lpfnGetShareString            GetShareString;
	static lpfnSetShareString            SetShareString;
	static lpfnGetIP                     GetIP;
	static lpfnGetIPString               GetIPString;
	static lpfnGetISPName                GetISPName;
	static lpfnGetOpStatus               GetOpStatus;
	static lpfnSetOpStatus               SetOpStatus;
	static lpfnGetLanguage               GetLanguage;
	static lpfnSetLanguage               SetLanguage;
	static lpfnGetMode                   GetMode;
	static lpfnSetMode                   SetMode;
	static lpfnModeTransform             ModeTransform;
	static lpfnGetMastersCount           GetMastersCount;
	static lpfnGetMastersIds             GetMastersIds;
	static lpfnGetIpCacheInfo            GetIpCacheInfo;
	static lpfnSetIpCacheInfo            SetIpCacheInfo;
	static lpfnGetAccessLevel            GetAccessLevel;
	static lpfnSetAccessLevel            SetAccessLevel;
	static lpfnGetRights                 GetRights;
	static lpfnSetRights                 SetRights;
	static lpfnGetProfileId              GetProfileId;
	static lpfnGetAccountId              GetAccountId;
	static lpfnCreateAccount             CreateAccount;
	static lpfnDeleteAccount             DeleteAccount;
	static lpfnMoveAccount               MoveAccount;
	static lpfnSetProfile                SetProfile;
	static lpfnGetProfile                GetProfile;
	static lpfnGetNextProfile            GetNextProfile;
	static lpfnGetAccount                GetAccount;
	static lpfnSetAccount                SetAccount;
	static lpfnGetNextAccount            GetNextAccount;
	static lpfnAllocUserData             AllocUserData;
	static lpfnFreeUserData              FreeUserData;
	static lpfnGetUserData               GetUserData;
	static lpfnSetUserData               SetUserData;
	static lpfnUnregisterCommand         UnregisterCommand;
	static lpfnUnregisterHelpString      UnregisterHelpString;
	static lpfnUnregisterUserCommand     UnregisterUserCommand;
	static lpfnRegisterUserCommand       RegisterUserCommand;
	static lpfnRegisterChatFilter        RegisterChatFilter;
	static lpfnUnregisterChatFilter      UnregisterChatFilter;
	static lpfnRegisterBot               RegisterBot;
	static lpfnUnregisterBot             UnregisterBot;
	static lpfnCallHubCommand            CallHubCommand;
	static lpfnProcessUserTraffic        ProcessUserTraffic;
	static lpfnDisableOutputToUser       DisableOutputToUser;
	static lpfnAcceptUser                AcceptUser;
	static lpfnGetTopic                  GetTopic;
	static lpfnSetTopic                  SetTopic;
	static lpfnGetMOTD                   GetMOTD;
	static lpfnSetMOTD                   SetMOTD;
	static lpfnGetHubUptime              GetHubUptime;
	static lpfnGetHublistEntry           GetHublistEntry;
	static lpfnRegisterBotEx             RegisterBotEx;

#if (HEXHUB_VERSION >= 320)
	static lpfnDisconnectUser            DisconnectUser;
#endif

	bool Init(lpfnGetFunction GetFunction);
};
#endif // defined(HAVE_HEXHUB_CPP_UTILS) && defined(__cplusplus)

#endif // HEXHUB_H